'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /* global Buffer */

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _RequestShortener = require('webpack/lib/RequestShortener');

var _RequestShortener2 = _interopRequireDefault(_RequestShortener);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var AssetMapPlugin = function () {
  /**
   * AssetMapPlugin
   *
   * @param {string} outputFile - Where to write the asset map file
   * @param {string} [relativeTo] - Key assets relative to this path, otherwise defaults to be relative to the directory where the outputFile is written
   */
  function AssetMapPlugin(outputFile, relativeTo) {
    _classCallCheck(this, AssetMapPlugin);

    this.outputFile = outputFile;
    this.relativeTo = relativeTo;
    this.previousChunks = {};
  }

  _createClass(AssetMapPlugin, [{
    key: '_extractAssets',
    value: function _extractAssets(modules, requestShortener, publicPath) {
      var emitted = false;
      var mappedAssets = modules.map(function (m) {
        var assets = Object.keys(m.assets || {});

        if (assets.length === 0) {
          return undefined;
        }

        var asset = assets[0];
        emitted = emitted || m.assets[asset].emitted;

        return {
          name: m.readableIdentifier(requestShortener),
          asset: asset
        };
      }).filter(function (m) {
        return m !== undefined;
      }).reduce(function (acc, m) {
        acc[m.name] = _url2.default.resolve(publicPath, m.asset);
        return acc;
      }, {});

      return [emitted, mappedAssets];
    }
  }, {
    key: '_extractChunks',
    value: function _extractChunks(chunks, publicPath) {
      var mappedChunks = chunks.map(function (c) {
        return {
          name: c.name,
          files: c.files.filter(function (f) {
            return _path2.default.extname(f) !== '.map';
          }).map(function (f) {
            return _url2.default.resolve(publicPath, f);
          })
        };
      }).reduce(function (acc, c) {
        acc[c.name] = c.files;
        return acc;
      }, {});

      var emitted = JSON.stringify(this.previousChunks) !== JSON.stringify(mappedChunks);
      this.previousChunks = mappedChunks;

      return [emitted, mappedChunks];
    }
  }, {
    key: 'apply',
    value: function apply(compiler) {
      var _this = this;

      compiler.plugin('emit', function (compilation, done) {
        var publicPath = compilation.outputOptions.publicPath;
        var requestShortener = new _RequestShortener2.default(_this.relativeTo || compilation.outputOptions.path);

        var _extractAssets2 = _this._extractAssets(compilation.modules, requestShortener, publicPath),
            _extractAssets3 = _slicedToArray(_extractAssets2, 2),
            assetsEmitted = _extractAssets3[0],
            assets = _extractAssets3[1];

        var _extractChunks2 = _this._extractChunks(compilation.chunks, publicPath),
            _extractChunks3 = _slicedToArray(_extractChunks2, 2),
            chunksEmitted = _extractChunks3[0],
            chunks = _extractChunks3[1];

        if (assetsEmitted || chunksEmitted) {
          var out = JSON.stringify({ assets: assets, chunks: chunks }, null, 2);
          compilation.assets[_this.outputFile] = {
            source: function source() {
              return out;
            },
            size: function size() {
              return Buffer.byteLength(out, 'utf8');
            }
          };
        }
        done();
      });
    }
  }]);

  return AssetMapPlugin;
}();

exports.default = AssetMapPlugin;